{ Component "WndListView V1.0" is a ListView for showing all windows.
  Extended window information can be retrieved by OnWindowChange. Updates
  windowlist by clicking column caption.

  Udo Juerss
  57078 Siegen, Germany
  April 1999

  e-mail: ujhs@aol.com
}

unit
  WndListView;

interface
{------------------------------------------------------------------------------}

uses
  Windows,Messages,Classes,ComCtrls,SysUtils;
{------------------------------------------------------------------------------}

type
  PWndRec = ^TWndRec;
  TWndRec = record
    Handle: HWnd;
    Name: string;
    ClassName: string;
    ParentName: string;
    ProcessID: Integer;
    Visible: Boolean;
    Enabled: Boolean;
    Iconic: Boolean;
    Rect: TRect;
  end;

  TWndNotify = procedure(Sender: TObject; Window: TWndRec) of object;

  TWndListView = class(TCustomListView)
  private
    WndList: TList;
    FCount: Integer;
    FOnWindowChange: TWndNotify;
    StartUp: Boolean;
  protected
    procedure WndProc(var Message: TMessage); override;
    function GetWindow(Index: Integer): TWndRec;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DoUpdate(Sender: TObject; Column: TListColumn);
    procedure DoWindowChange(Sender: TObject; Item: TListItem; Change: TItemChange);
    property Windows[Index: Integer]: TWndRec read GetWindow;
    property Count: Integer read FCount;
  published
    property OnWindowChange: TWndNotify read FOnWindowChange write FOnWindowChange;

//  predefined properties (add those you want to publish too)
    property Align;
    property AllocBy;
    property Anchors;
    property FlatScrollBars;
    property Font;
    property Hint;
    property PopupMenu;
    property ShowHint;

//    property OnChange; internally assigned and redirected to OnWindowChange
//    property OnColumnClick; internally assigned and redirected to DoUpdate
  end;
{------------------------------------------------------------------------------}

procedure Register;

implementation
{------------------------------------------------------------------------------}

function WindowCallback(WHandle: HWnd; var PWndList: Pointer): Bool; stdcall;
var
  PStr: PChar;
  Int: Integer;
  WinPtr: PWndRec;
begin
  New(WinPtr);
  WinPtr.Handle:=WHandle;
  PStr:=AllocMem(255);
  GetWindowText(WHandle,PStr,255);
  WinPtr.Name:=string(PStr);
  GetClassName(WHandle,PStr,255);
  WinPtr.ClassName:=string(PStr);
  GetWindowText(GetParent(WHandle),PStr,255);
  WinPtr.ParentName:=string(PStr);
  FreeMem(PStr,255);
  Int:=0;
  WinPtr.ProcessID:=GetWindowThreadProcessId(WHandle,@Int);
  WinPtr.Visible:=IsWindowVisible(WHandle);
  WinPtr.Enabled:=IsWindowEnabled(WHandle);
  WinPtr.Iconic:=IsIconic(WHandle);
  WinPtr.Rect:=Rect(0,0,0,0);
  GetWindowRect(WHandle,WinPtr.Rect);

  TList(PWndList).Add(WinPtr);
  Result:=True;
end;
{------------------------------------------------------------------------------}

constructor TWndListView.Create(AOwner: TComponent);
var
  LC: TListColumn;
begin
  inherited;
  WndList:=TList.Create;
  EnumWindows(@WindowCallback,LongInt(@WndList));
  FCount:=WndList.Count;

  AllocBy:=SizeOf(TWndRec);
  ViewStyle:=vsReport;
  RowSelect:=True;
  ReadOnly:=True;
  OnColumnClick:=DoUpdate;
  OnChange:=DoWindowChange;
  ShowHint:=True;
  Hint:='Click column caption to update';
  GridLines:=True;

  Columns.Clear;
  LC:=Columns.Add;
  LC.Caption:='Handle';
  LC.Width:=50;
  LC:=Columns.Add;
  LC.Caption:='Name';
  LC.Width:=150;
  LC:=Columns.Add;
  LC.Caption:='Class';
  LC.Width:=250;

  StartUp:=True;
end;
{------------------------------------------------------------------------------}

destructor TWndListView.Destroy;
var
  I: Integer;
begin
  if WndList.Count > 0 then for I:=0 to Pred(WndList.Count) do
    Dispose(PWndRec(WndList[I]));
  WndList.Free;
  inherited;
end;
{------------------------------------------------------------------------------}

procedure TWndListView.WndProc(var Message: TMessage);
begin
  inherited;
  if (Message.Msg = WM_PAINT) and StartUp then
  begin
    StartUp:=False;
    DoUpdate(nil,nil);
  end;
end;
{------------------------------------------------------------------------------}

procedure TWndListView.DoUpdate(Sender: TObject; Column: TListColumn);
var
  N: Integer;
  LI: TListItem;
begin
  WndList.Clear;
  WindowCallBack(GetDesktopWindow,Pointer(WndList));
  EnumWindows(@WindowCallBack,LongInt(@WndList));
  FCount:=WndList.Count;

  Items.Clear;
  for N:=0 to Pred(FCount) do
  begin
    LI:=Items.Add;
    LI.Caption:=(IntToStr(Windows[N].Handle));
    LI.SubItems.Add(Windows[N].Name);
    LI.SubItems.Add(Windows[N].ClassName);
  end;
end;
{------------------------------------------------------------------------------}

procedure TWndListView.DoWindowChange(Sender: TObject; Item: TListItem; Change: TItemChange);
begin
  if Assigned(FOnWindowChange) then FOnWindowChange(Self,Windows[Item.Index]);
end;
{------------------------------------------------------------------------------}

function TWndListView.GetWindow(Index: Integer): TWndRec;
begin
  Result:=PWndRec(WndList[Index])^;
end;
{------------------------------------------------------------------------------}

procedure Register;
begin
  RegisterComponents('Udo',[TWndListView]);
end;
{------------------------------------------------------------------------------}

initialization
end.

